function drawSVG(target, name, attributes, html) {
	var item = document.createElementNS('http://www.w3.org/2000/svg', name);
	for (var key in attributes)
		item.setAttribute(key, attributes[key]);
	if (html != null)
	{
		if (name == 'text')
			item.textContent = html;
		else
			item.innerHTML = html;
	}
	if (target != null)
		target.appendChild(item);
	return item;
}

function showGraph(svg, scanApFreq, scanApData) {
	var w 			= svg.getAttribute('width');
	var h			= svg.getAttribute('height');
	var pad			= [60, 10, 30, 50 ];			// Left, right, top, bottom
	var s_width		= 5;
	var r_power		= [0, 100, 10];					// Min, Max, Step
	var chan_sh		= [ 10, 10 ];					// Left, right

	if (scanApFreq == "2.4") {
		var z		= 10;
		var chans	= [ 2412, 2417, 2422, 2427, 2432, 2437, 2442, 2447, 2452, 2457, 2462, 2467, 2472, 2484 ];	// 2.4GHz channels freqency
	}
	else {
		var z		= 3;
		var chans	= [ [5160, 32], [5180, 36], [5200, 40], [5220, 44], [5240, 48], [5260, 52], [5280, 56], [5300, 60], [5320, 64],
						[5480, 96], [5500, 100], [5520, 104], [5540, 108], [5560, 112], [5580, 116], [5600, 120], [5620, 124], [5640, 128], [5660, 132], [5680, 136], [5700, 140], [5720, 144],
						[5725, 145], [5745, 149], [5765, 153], [5785, 157], [5805, 161], [5825, 165]
					  ];							// 5GHz channels freqency
	}
	// Background
	drawSVG(svg, 'rect',
		{
			'fill':				'#363636',
			'fill-opacity':		'1',
			'fill-rule':		'evenodd',
			'stroke':			'none',
			'x':				'0',
			'y':				'0',
			'width':			w,
			'height':			h
		}
	);

	// Show channels
	for (var chan = 0; chan < chans.length; chan++)
	{
		if (scanApFreq == "2.4") {
			var dist = chans[chans.length-1] - chans[0] + chan_sh[0] + chan_sh[1];
			var x = Math.round(pad[0] + (w - pad[0] - pad[1]) * ((chans[chan] - chans[0] + chan_sh[0])/dist));
		}
		else {
			var shift = (chans[chan][1] <= 64) ? 0 : (chans[chan][1] <= 144) ? 20 : 40;
			var x = Math.round(pad[0] + chan_sh[0] + shift + ((w - pad[0] - 40) / (chans.length + 2)) * (chan + 1));
		}

		drawSVG(svg, 'line',
			{
				'x1':				x,
				'y1':				pad[2],
				'x2':				x,
				'y2':				h - pad[3] + s_width + (chan % 2) * 15,
				'style':			'stroke:#a0a0a0;',
				'fill':				'none',
				'stroke-dasharray': '2,2'
			}
		);

		// Channel
		drawSVG(svg, 'text',
			{
				'fill':					'#c0c0c0',
				'x':					x,
				'y':					pad[2] - 5,
				'text-anchor':			'middle',
				'dominant-baseline':	'text-after-edge',
				'text-rendering':		'optimizeLegibility',
				'font-size':			'8pt',
				'font-family':			'Tahoma',
				'fill-rule':			'evenodd'
			},
			(scanApFreq == "2.4") ? chan + 1 : chans[chan][1]
		);

		// Frequence
		drawSVG(svg, 'text',
			{
				'fill':					'#c0c0c0',
				'x':					x,
				'y':					h - pad[3] + s_width + 5 + (chan % 2)*15,
				'dominant-baseline':	'text-before-edge',
				'text-anchor':			'middle',
				'text-rendering':		'optimizeLegibility',
				'font-size': 			'8pt',
				'font-family':			'Tahoma',
				'fill-rule': 			'evenodd'
			},
			(scanApFreq == "2.4") ? chans[chan] : chans[chan][0]
		);
	}

	// Draw horizontal grid
	for (var power = r_power[0]; power <= r_power[1]; power += r_power[2])
	{
		var dist	= r_power[0] + r_power[1];
		var red		= Math.round(255 - 255 * (power / dist));
		var green	= Math.round(255 - 255 * ((dist - power) / dist));
		var y		= Math.round(h - 20 - (pad[2] + (h - pad[2] - pad[3]) * (power / 110)));

		drawSVG(svg, 'line',
			{
				'x1': 					pad[0] - s_width,
				'y1': 					y,
				'x2': 					w - pad[1],
				'y2': 					y,
				'style': 				'stroke:rgb(' + red + ',' + green +',0);',
				'stroke-dasharray': 	'2,2'
			}
		);

		// Signal
		drawSVG(svg, 'text',
			{
				'fill':					'rgb(' + red + ', ' + green +', 0)',
				'x':					pad[0] - s_width - 5,
				'y':					y,
				'text-anchor':			'end',
				'dominant-baseline':	'central',
				'text-rendering':		'optimizeLegibility',
				'font-size':			'8pt',
				'font-family':			'Tahoma',
				'fill-rule':			'evenodd'
			},
			power
		);
	}

	// Draw stations
	for (var i = 0; i < scanApData.length; i++) {
		var dist	= chans[chans.length-1] - chans[0] + chan_sh[0] + chan_sh[1];
		var dx		= w - pad[0] - pad[1];
		var dy		= h - pad[2] - pad[3];
		//var thick	= (scanApData[i].signal >= 66) ? 4 : (scanApData[i].signal > 33) ? 3 : 2;
		var thick	= 3;

		if (scanApFreq == "2.4") {
			switch (scanApData[i].extch) {
				case 'NONE':
					var x1 = Math.round(pad[0] + dx * ((chans[+scanApData[i].channel - 1] - chans[0] + chan_sh[0] - 10)/dist));
					var x2 = Math.round(pad[0] + dx * ((chans[+scanApData[i].channel - 1] - chans[0] + chan_sh[0] + 10)/dist));
					break;
				case 'ABOVE':
					var x1 = Math.round(pad[0] + dx * ((chans[+scanApData[i].channel - 1] - chans[0] + chan_sh[0] - 10) / dist));
					var x2 = Math.round(pad[0] + dx * ((chans[+scanApData[i].channel + 4 - 1] - chans[0] + chan_sh[0] + 10) / dist));
					break;
				case 'BELOW':
					var x1 = Math.round(pad[0] + dx * ((chans[+scanApData[i].channel - 4 - 1] - chans[0] + chan_sh[0] - 10) / dist));
					var x2 = Math.round(pad[0] + dx * ((chans[+scanApData[i].channel - 1] - chans[0] + chan_sh[0] + 10) / dist));
					break;
			}
		}
		else {
			for (var j = 0; j < chans.length; j++)
				if (chans[j][1] == scanApData[i].channel) {
					var shift = (chans[j][1] <= 64) ? 0 : (chans[j][1] <= 144) ? 20 : 40;
					switch (scanApData[i].extch) {
						case 'NONE':
							var x1 = Math.round(pad[0] + chan_sh[0] + shift + (w - pad[0] - 40) / (chans.length + 2) * j + (w - pad[0] - 40) / (chans.length + 2) * 0.5);
							var x2 = Math.round(pad[0] + chan_sh[0] + shift + (w - pad[0] - 40) / (chans.length + 2) * (j + 2) - (w - pad[0] - 40) / (chans.length + 2) * 0.5);
							break;
						case 'ABOVE':
							var x1 = Math.round(pad[0] + chan_sh[0] + shift + (w - pad[0] - 40) / (chans.length + 2) * j + ((w - pad[0] - 40) / (chans.length + 2)) * 0.5);
							var x2 = Math.round(pad[0] + chan_sh[0] + shift + (w - pad[0] - 40) / (chans.length + 2) * (j + 3) - (w - pad[0] - 40) / (chans.length + 2) * 0.5);
							break;
						case 'BELOW':
							var x1 = Math.round(pad[0] + chan_sh[0] + shift + (w - pad[0] - 40) / (chans.length + 2) * (j - 1) + (w - pad[0] - 40) / (chans.length + 2) * 0.5);
							var x2 = Math.round(pad[0] + chan_sh[0] + shift + (w - pad[0] - 40) / (chans.length + 2) * (j + 2) - (w - pad[0] - 40) / (chans.length + 2) * 0.5);
							break;
					}
				}
		}
		var y		= Math.round(pad[2] + dy * ((110 - scanApData[i].signal) / (r_power[2] - r_power[0] + r_power[1])));

		var col		= scanApData[i].color;

		drawSVG(svg, 'line',
			{
				'x1':		x1 + z,
				'y1':		y,
				'x2':		x1,
				'y2':		h - pad[3],
				'style':	'stroke-width: ' + thick + '; stroke: #' + col + '; stroke-opacity: 0.65;'
			}
		);
		drawSVG(svg, 'line',
			{
				'x1':		x1 + z,
				'y1':		y,
				'x2':		x2 - z,
				'y2':		y,
				'style':	'stroke-width: ' + thick + '; stroke: #' + col + '; stroke-opacity: 0.65;'
			}
		);
		drawSVG(svg, 'line',
			{
				'x1':		x2 - z,
				'y1':		y,
				'x2':		x2,
				'y2':		h - pad[3],
				'style':	'stroke-width: ' + thick + '; stroke: #' + col + '; stroke-opacity: 0.65;'
			}
		);
	}

	// Draw text for stations
	for (var i = 0; i < scanApData.length; i++) {
		if (scanApFreq == "2.4") {
			switch (scanApData[i].extch) {
				case 'NONE':
					var x = Math.round(pad[0] + dx * ((chans[+scanApData[i].channel - 1] - chans[0] + chan_sh[0]) / dist));
					break;
				case 'ABOVE':
					var x = Math.round(pad[0] + dx * ((chans[+scanApData[i].channel + 2 - 1] - chans[0] + chan_sh[0]) / dist));
					break;
				case 'BELOW':
					var x = Math.round(pad[0] + dx * ((chans[+scanApData[i].channel - 2 - 1] - chans[0] + chan_sh[0]) / dist));
					break;
			}
		}
		else {
			for (var j = 0; j < chans.length; j++)
				if (chans[j][1] == scanApData[i].channel) {
					var shift = (chans[j][1] <= 64) ? 0 : (chans[j][1] <= 144) ? 20 : 40;
					switch (scanApData[i].extch) {
						case 'NONE':
							var x = Math.round(pad[0] + chan_sh[0] + shift + (w - pad[0] - 40) / (chans.length + 2) * (j + 1));
							break;
						case 'ABOVE':
							var x = Math.round(pad[0] + chan_sh[0] + shift + (w - pad[0] - 40) / (chans.length + 2) * (j + 2) - (w - pad[0] - 40) / (chans.length + 2) * 0.5);
							break;
						case 'BELOW':
							var x = Math.round(pad[0] + chan_sh[0] + shift + (w - pad[0] - 40) / (chans.length + 2) * j + (w - pad[0] - 40) / (chans.length + 2) * 0.5);
							break;
					}
				}
		}
		var y = Math.round(pad[2] + dy * ((110 - scanApData[i].signal) / (r_power[2] - r_power[0] + r_power[1])));
		var col = scanApData[i].color;

		if (scanApData[i].ssid[0] == "0" && scanApData[i].ssid[1] == "x")
			scanApData[i].ssid = convertHexToString(scanApData[i].ssid.substr(2, scanApData[i].ssid.length - 2));

		drawSVG(svg, 'text',
			{
				'fill':					'#' + col,
				'x':					x,
				'y':					y - 5,
				'text-anchor':			'middle',
				'dominant-baseline':	'text-after-edge',
				'text-rendering':		'optimizeLegibility',
				'font-size':			'10pt',
				'font-family':			'Tahoma',
				'fill-rule':			'evenodd',
				'font-weight':			'bold'
			},
			(scanApData[i].ssid.length > 0) ? scanApData[i].ssid : scanApData[i].bssid
		);
	}

	// Vertical line
	drawSVG(svg, 'line', { 'x1': pad[0], 'y1': pad[2], 'x2': pad[0], 'y2': h - pad[3] + s_width, 'style': 'stroke: #909090; stroke-width: 2px;', 'fill': 'none' } );
	// Horizontal line
	drawSVG(svg, 'line', { 'x1': pad[0] - s_width, 'y1': h - pad[3], 'x2': w - pad[1], 'y2': h - pad[3], 'style': 'stroke: #909090; stroke-width: 2px;', 'fill': 'none' } );

	// Draw text
	drawSVG(svg, 'text',
		{
			'fill':					'#c0c0c0',
			'x':					0,
			'y':					0,
			'text-anchor':			'middle',
			'dominant-baseline':	'central',
			'transform':			'rotate(-90), translate(' + (-h/2) + ', 15)',
			'text-rendering':		'optimizeLegibility',
			'font-size':			'10pt',
			'font-family':			'Tahoma',
			'fill-rule':			'evenodd'
		},
		'Signal [%]'
	);
}

function scanAP(ap)	{
	new Ext.window.Window({
			width:			800,
			height:			600,
			title:			'Scan AP',
			id:				'scanap_window',
			shadow:			true,
			resizable:		false,
			draggable:		true,
			closable:		false,
			autoShow:		true,
			modal:			true,
			items:			[
								{
									border:	false,
									html:	'<div id="scanAPsvg" style="width: 788px; height: 360px; background-color: #dfe8f6; min-height: 12em; display: table-cell; vertical-align: middle;"></div>',
								},
								{
									border:			false,
									xtype:			'grid',
									id:				'scanap_legend_grid',
									height:			175,
									store:			{
														fields: 		[
																			{	name: 'channel',	type: 'string'	},
																			{	name: 'extch',		type: 'string'	},
																			{	name: 'ssid',		type: 'string'	},
																			{	name: 'bssid',		type: 'string'	},
																			{	name: 'security',	type: 'string'	},
																			{	name: 'signal',		type: 'int'		},
																			{	name: 'mode',		type: 'string'	},
																			{	name: 'type',		type: 'string'	},
																			{	name: 'color',		type: 'string'	}
																		],
														autoLoad: 		false,
														remoteFilter:	true,
														remoteSort:		false,
														proxy: 			{
																			type:			'ajax',
																			url: 			'server/get_apscan.php',
																			actionMethods:	{
																								create:		'POST',
																								read:		'POST',
																								update: 	'POST',
																								destroy:	'POST'
																							},
																			reader:			{	type:		'json'	}
																		}
													},
									viewConfig:		{
														enableTextSelection:	true,
														loadMask:				false,
													},
						columns: 		[
											{	flex: 5 / 100,
												renderer:	function (value, meta, record, rowIndex) {
																meta.style = "background-color: #" + record.data.color + ";";
																return '';
															}
											},
											{	text: 'Channel',	dataIndex: 'channel',	flex: 8 / 100,
												align: 'center',
												renderer:	function (value, meta, record, rowIndex) {
																if (record.data.extch == 'ABOVE')
																	return value + ' + ' + (+value + 4);
																else if (record.data.extch == 'BELOW')
																	return value + ' + ' + (+value - 4);
																else
																	return value;
															}
											},
											{	text: 'SSID',		dataIndex: 'ssid',		flex: 22 / 100	}, 
											{	text: 'BSSID',		dataIndex: 'bssid', 	flex: 16 / 100	},
											{	text: 'Security',	dataIndex: 'security', 	flex: 23 / 100	},
											{	text: 'Signal (%)',	dataIndex: 'signal',	flex: 10 / 100,	align: 'center'	},
											{	text: 'Mode',		dataIndex: 'mode',		flex: 8 / 100	},
											{	text: 'Type',		dataIndex: 'type',		flex: 8 / 100	},
										],
								}
							],
			listeners:		{
								afterrender:	function () {
													var item = grid_AP_panel.getStore().getAt(grid_AP_panel.store.find('sqlid', ap));
													if (item !== undefined)
														try { var model = item.data.host_model.split(" ")[0] }
														catch(e) { var model = ''};
													if (model == 'SNR-CPE-W2N-MT' ||
														model == 'SNR-CPE-W4N-MT' ||
														model == 'SNR-CPE-WI2-MT' ||
														model == 'SNR-CPE-AP1-MT' ||
														model == 'SNR-CPE-SFP-MT') {
															Ext.getCmp('scanap_scan5_button').hide();
															Ext.getCmp('scanap_scan24_button').setText('Scan');
														}
												}
							},
			buttons:		[
								{ 
									text: 		'Scan 2.4GHz',
									id:			'scanap_scan24_button',
									handler:	function() { getScanAP('2.4'); }
								},
								{ 
									text: 		'Scan 5GHz',
									id:			'scanap_scan5_button',
									handler:	function() { getScanAP('5'); }

								},
								{ 
									text: 		'Close',
									id:			'scanap_close_button',
									handler:	function() {
													Ext.getCmp('scanap_window').close();
												}
								}
							]
	});
	
	function getScanAP(band) {
		// Clearing previous scan
		var div = document.getElementById('scanAPsvg');
		while (div.firstChild)
			div.removeChild(div.firstChild);

		// Show preloader
		div.innerHTML = '<div style="text-align: center; color: #505050;">Please wait, it will take about 30 seconds.</div><img src="/images/preloader.gif" class="center" />';
		
		// Clearing Legend
		Ext.getCmp('scanap_legend_grid').store.removeAll();

		// Disable buttons
		Ext.getCmp('scanap_scan24_button').disable();
		Ext.getCmp('scanap_scan5_button').disable();
		Ext.getCmp('scanap_close_button').disable();

		var request		= {};
		request.id		= ap;
		request.band	= band;

		$.post('server/get_apscan.php', request, 
			function(data) {
				// Disable buttons
				Ext.getCmp('scanap_scan24_button').enable();
				Ext.getCmp('scanap_scan5_button').enable();
				Ext.getCmp('scanap_close_button').enable();

				// Get data
				try { var result = JSON.parse(data); } catch (e) { var result = JSON.parse(JSON.stringify(data)); }

				// Remove preloader
				while (div.firstChild)
					div.removeChild(div.firstChild);

				if (result.state == 'error')
					Ext.MessageBox.show({
						title:		'ScanAP error!',
						msg:		'status: ' + result.state + '<br>response: ' + result.response,
						buttons:	Ext.MessageBox.OK,
					});
				else {
					// Draw graph
					var el = drawSVG(div, 'svg', { 'width': 788, 'height': 360 });
					showGraph(el, band, result.data);

					// Show legend
					Ext.getCmp('scanap_legend_grid').store.loadData(result.data);
				}
			}
		);
	}
}